/*
	Graphic Workshop Plugin Format
*/

#define	PLUGIN_VERSION		15
#define	PLUGIN_SUBVERSION	1

#define	PLUGINVERSION()		((PLUGIN_VERSION << 8) | PLUGIN_SUBVERSION)

#define	PLUGIN_FOLDER		COMMON_GRAPHIC_PLUGIN_FOLDER

#ifndef _WIN64
#define	ULONG_PTR		ULONG
#endif

// parameter structure for plugins
typedef struct {
	unsigned long size;			//sizeof(GWSPLUGINPARAM)
        unsigned long version;			//plugin version -- must be =PLUGINVERSION()
        unsigned long magic;
        char copyright[261];			//a copyright string, for the credits window
	char extension[261];        		//the format file extension - can be up to 32 extensions separated by semicolons
        char description[261];			//a GWSPRO description of the format
        unsigned long maxbits;			//the maximum number of bits of colour supported by the format - up to 24
        unsigned long flags;			//array of FT_ flags, below
        ULONG_PTR hinstance;		        //instance of calling application
        unsigned long reserved[14];		//reserved, must be set to zero
 	} GWSPLUGINPARAM;

#define	PLUGINCOMMENTSIZE	64

// info structure for plugins
typedef struct {
	unsigned long size;			//sizeof(GWSPLUGINFILEINFO)
        unsigned long version;			//plugin version -- must be =PLUGINVERSION()
	unsigned long magic;
	unsigned long width;			//width of image, in pixels
        unsigned long depth;			//depth of image, in pixels
        unsigned long reserved1;
        unsigned long bits;			//bits of colour
        unsigned long reserved2;
	unsigned long flags;			//flags -- currently unused, must be set to zero
        unsigned long reserved3;
	unsigned long background;		//optional background colour. This should be an RGB colour if bits > 8, or a palette index otherwise
	unsigned long filesize;			//the size in bytes of the packed file
	unsigned long imagesize;		//the size in bytes of the unpacked image
	char palette[768];			//the colour palette, if there is one
	unsigned long pagecount;		//the number of pages in the document file
	unsigned long reserved[15];		//reserved, must be set to zero
        char comments[PLUGINCOMMENTSIZE+1];	//comment field
	} GWSPLUGINFILEINFO;

typedef struct {
	unsigned long size;			//sizeof(GWSPLUGINCALLBACK)
        unsigned long version;			//plugin version -- must be =PLUGINVERSION()
       	unsigned long reserved1;
        unsigned long magic;			//magic identification number GWSPLUGINCALLBACK_MAGIC
        WPARAM wParam;				//NULL
        LPARAM lParam;				//NULL
       	unsigned long reserved2;
	int (*isregistered)();			//proc returns true if Graphic Workshop is registered
        int (*validate)(WPARAM wParam,LPARAM lParam);	//optional validation proc - do not use in evergreen plugins
	void (*openstatus)();			//proc to open the status bar
        void (*updatestatus)(unsigned int cur,unsigned int max);	//proc to update thew status bar
        void (*closestatus)();			//proc to close the status bar
        void (*statustext)(char *string);	//proc to set the status text
        void *configitem;			//points to configitem[0] - do not use in evergreen plugins
        int configcount;			//number of config items - do not use in evergreen plugins
        long pagenumber;			//page in multiple image file to read
	} GWSPLUGINCALLBACK;

typedef struct {
	unsigned long size;			//sizeof(GWSPLUGINCONFIGPARAM)
        unsigned long version;			//plugin version -- must be =PLUGINVERSION()
       	unsigned long reserved1;
        unsigned long magic;			//magic identification number GWSPLUGINCALLBACK_MAGIC
        int (*isregistered)();			//proc returns true if Graphic Workshop is registered
	LRESULT (*drawcolorbutton) (HWND hwnd,UINT message,WPARAM wParam,LPARAM lParam,unsigned long colour,unsigned long flags); //draws an owner draw Graphic Workshop color button in a configuration window - set flags to zero for evergreen plugins
        int (*selectcolor)(HWND hwnd,LPCOLORREF color);		//opens a Graphic Workshop color selection window
        int (*dohelp) (HWND hwnd,char *document,char *key);	//opens the help for your plugin - see the SDK documentation
        WPARAM wParam;				//NULL
        LPARAM lParam;				//NULL
        void *pr;
        int prc;
	} GWSPLUGINCONFIGPARAM;

#define GWSPLUGINCALLBACK_MAGIC			'RXaR'

// note -- the version item must be set to PLUGINVERSION() before calling PLUGINGetPluginParams to tell the plugin which version of the
// toolkit Graphic Workshop is using. It will return set to the version of the toolkit used to create the plugin.

//flags for plugins to tell Graphic Workshop what sort of support the plugin provides

#define	FT_BITMAP	0x0001		//the plugin handles bitmap images -- pretty much essential
#define	FT_VECTOR	0x0002		//the plugin returns a handle to a metafile -- currently unsupported
#define	FT_MULTIPLE	0x0004		//the plugin handles a format with multiple image capability
#define	FT_ANIMATION	0x0008		//the plugin handles a format with animation capability  -- currently unsupported
#define	FT_MSIZE	0x0100 		//the plugin handles a format with multiple image capability in which the images may be of differing sizes  -- currently unsupported
#define	FT_READONLY	0x10000		//the plugin can read images, but not write them
#define	FT_WRITEONLY	0x20000		//the plugin write images, but not read them
#define	FT_NOANIMCNVT	0x40000		//the plugin handles a format with animation but not animation conversion  -- currently unsupported
#define	FT_SUPPORTALPHA	0x2000000       //the plugin supports alpha


//status codes for plugin operations

#define	R_SUCCESS		0	//the operation was successful
#define	BAD_OPEN		1	//the file could not be opened
#define	BAD_FILE		2	//the file is corrupt
#define	BAD_READ		3	//there was an error reading the file
#define	CANCEL_FUNCTION		4	//the operation was cancelled
#define	UNSUPPORTED_FUNCTION	5	//the function is unsupported
#define	BAD_ALLOC		6	//there was an error allocating memory
#define	INFORMATION_ERROR	7	//there was an error getting information about the file
#define	UNSUPPORTED_FORMAT	8	//the format of the file is unsupported
#define	BAD_CREATE		10	//the file could not be created for writing
#define	BAD_WRITE		11	//the file could not be written to
#define	TOO_MANY_BITS		12	//the file had more bits of colour than the plugin could handle
#define	UNSUITABLE_IMAGE	13	//the image was unsuitable -- no one knows why
#define	RES_ERROR		14	//internal resource error
#define	INVALID_DLL		15	//bad library
#define	NO_BITMAP		20	//no image could be found in the file
#define	DISABLED_FEATURE	22	//a feature required to process the file or image has been disabled

#define PLUGIN_EXTENSION_MAX    32	//the maximum number of extensions per plugin

#ifdef _MSC_BUILD
//visual studio DLL C-style export
#define	DLL_EXPORT(TYPE)	extern "C" __declspec(dllexport) TYPE
#else
//if you're not using Visual Studio, you'll probably want to modify this DLL C-style export
#define	DLL_EXPORT(TYPE)	extern "C" WINAPI _export TYPE
#endif

#define	INISECTIONGWS				COMMON_GWS_INISECTION
#define	INIFILEGWS				COMMON_GWS_INI

#define	APPDATA_SUBFOLDER			COMMON_GWS_APPFOLDER

#define	EXPORTEG_GetVersion			PLUGIN_GetVersion
#define	EXPORTEG_GetCopyright			PLUGIN_GetCopyright
#define	EXPORTEG_GetPluginParams		PLUGIN_GetPluginParams
#define	EXPORTEG_ReadImage			PLUGIN_ReadImage
#define	EXPORTEG_WriteImage			PLUGIN_WriteImage
#define	EXPORTEG_GetInfo			PLUGIN_GetInfo
#define	EXPORTEG_GetComment			PLUGIN_GetComment
#define	EXPORTEG_DoConfigure			PLUGIN_DoConfigure

DLL_EXPORT (int) PLUGINGetVersion(void);
DLL_EXPORT (LPSTR) PLUGINGetCopyright(void);
DLL_EXPORT (int) PLUGINGetPluginParams(GWSPLUGINPARAM *param,LPARAM lParam);
DLL_EXPORT (HANDLE) PLUGINReadImage(char *path,int *result,LPARAM lParam);
DLL_EXPORT (int) PLUGINWriteImage(char *path,HANDLE handle,HANDLE comment,int *result,LPARAM lParam);
DLL_EXPORT (int) PLUGINGetInfo(GWSPLUGINFILEINFO *info,char *path,int *result,LPARAM lParam);
DLL_EXPORT (HANDLE) PLUGINGetComment(char *path,int *result,LPARAM lParam);
DLL_EXPORT (int) PLUGINDoConfigure(HWND hwnd,LPARAM lParam);




